import sys, os
from PIL import Image

def merge_images(file1, file2, stack_in_height=False):
    """Merge two images into one, displayed side by side
    :param file1: path to first image file
    :param file2: path to second image file
    :return: the merged Image object
    """
    image1 = Image.open(file1)
    image2 = Image.open(file2)

    (width1, height1) = image1.size
    (width2, height2) = image2.size

    if stack_in_height:
        result_width = max(width1,width2)
        result_height = height1 + height2
        start_height2 = height1
        start_width2 = 0
    else:
        result_width = width1 + width2
        result_height = max(height1, height2)
        start_height2 = 0
        start_width2 = width1

    result = Image.new('RGB', (result_width, result_height))
    result.paste(im=image1, box=(0, 0))
    result.paste(im=image2, box=(start_width2, start_height2))
    return result

if len(sys.argv) != 5:
    raise Exception("parsing input arguments failure %s" % sys.argv)
ndvi = sys.argv[1]
npp = sys.argv[2]
out_png = sys.argv[3]
roi = sys.argv[4]

stack_in_height = False     #stick images next to each other
if roi in ['SSB']:
    stack_in_height = True  #stick images above each other

merged  = merge_images(ndvi, npp, stack_in_height=stack_in_height)
merged.save( out_png )