import numpy.ma as ma
import numpy as np
import os
import re
import rasterio
import datetime
import subprocess
import datetime
import calendar

from frame_s2.config.FRAME_settings import meteo_settings_dict, lv2_settings

class Spectral_file():
    def __init__(self, name=None, Vlo=None, Vhi=None, Vint=None, Vslo=None, datatype=None, description=None, flags=None, units=None):
        self.name = name
        self.Vlo = Vlo     #max-value
        self.Vhi = Vhi     #min-value
        self.Vint = Vint    #offset
        self.Vslo = Vslo    #scale
        self.datatype = datatype
        self.description = description
        self.flags = flags
        self.units = units

def create_quicklook(in_file, colormap, scale=None, out_file=None, height=0, width=0):
    os.environ['GDAL_CACHEMAX'] = lv2_settings['GDAL_CACHEMAX']
    os.environ['GDAL_PAM_ENABLED'] = 'NO' # prevent creating aux.xml files
    if not os.path.exists(in_file):
        raise Exception("Unable to create quicklook for file %s, it does not exist." % in_file)
    if out_file is None:
        out_file = in_file.replace("tif",'png')
    tmp_file = in_file.replace(".tif",'_tmp.tif')

    if not os.path.exists(colormap):
        raise Exception ("Colormap-file does not exist: %s" % colormap)

    #resample image
    if scale is not None:
        scale_str = "-scale %s" %scale
    else:
        scale_str = ""

    print("Creating subsampled png file")
    cmd = "gdalwarp -overwrite --config GDAL_CACHEMAX %s -ts %s %s %s %s" % (lv2_settings['GDAL_CACHEMAX'], width, height, in_file, tmp_file)
    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        raise Exception("%s\n%s\n%s"  % (stdout, stderr, err_nr))
    tmp_png_file = in_file.replace("tif", 'png_tmp')
    cmd = "gdal_translate -ot Byte %s -of PNG %s %s" % ( scale_str, tmp_file, tmp_png_file)
    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        raise Exception("%s\n%s\n%s" % (stdout, stderr, err_nr))
    #adding color to final quicklook image
    print ("Add some color the the png file")
    cmd= "gdaldem color-relief %s %s %s" % (tmp_png_file, colormap, out_file)
    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        raise Exception("%s\n%s\n%s" ) % (stdout, stderr, err_nr)
    os.remove(tmp_file)
    os.remove(tmp_png_file)

def rescale(in_arr, offset_out, scale_out, max_value_in=250, min_value_in=0, max_value_out=250, min_value_out=0, offset_in=0, scale_in=1, out_data_type=np.ubyte):
    # rescale array with new scale and offset
    flags = np.where((in_arr > max_value_in) | (in_arr < min_value_in), in_arr, 0)

    # Keep track of saturation, using DIGITAL values!!!
    in_arr = ma.minimum(in_arr, max_value_out)  #flagged values while we overwritten again in the last step
    in_arr = ma.maximum(in_arr, min_value_out)  #keep track of min saturation

    physical_value = offset_in + in_arr * scale_in   #rescale values
    out_array = offset_out + scale_out * physical_value

    if out_data_type in [np.ubyte, np.int16]:
        out_array = np.floor(out_array + 0.5).astype(out_data_type)    # use rounding half up method
    out_array = np.where(flags==0,  out_array, flags)
    return out_array

def getDekadLst(startdekad, enddekad):
    dekad = str(startdekad)
    dekad_lst = []
    while dekad <= str(enddekad):
        dekad_lst.append(dekad)
        dekad=nextDekad(dekad)

    return dekad_lst
def getHistoricalDekadList(last_dekad, nrOfDekads2goBack):
    dekad_list=[str(last_dekad)]
    dekad=last_dekad
    for i in range(1, nrOfDekads2goBack):
        dekad = previousDekad(dekad)
        dekad_list.append(dekad)
    return dekad_list


def previousDekad(dekad):
    dekad = str(dekad)
    year = int("20" + dekad[0:2])
    dekadnr = int(dekad[2:4])
    if dekadnr == 1:
        year = year - 1
        dekadnr = 36
    else:
        dekadnr -= 1
    previousdekad = str(year)[2:4] + "{:02d}".format(dekadnr)
    return previousdekad

def nextDekad(dekad):
    dekad = str(dekad)
    year = int("20" + dekad[0:2])
    dekadnr = int(dekad[2:4])
    if dekadnr == 36:
        year = year + 1
        dekadnr = 1
    else:
        dekadnr += 1
    nextdekad = str(year)[2:4] + "{:02d}".format(dekadnr)
    return nextdekad


# --------------------------------------------------------------------------
## get_dec_start
#   Returns the first day of a decade
#   @author FC 16/12/10
#
#   @param[in] in_date YYYYMMDD
#   @return date YYYYMMDD (dd=decade start day)
# --------------------------------------------------------------------------
def get_dec_start(in_date):
    year = str(in_date)[0:4]
    month = str(in_date)[4:6]
    day = int(str(in_date)[6:8])

    if day < 11:
        dec_start_day = '01'
    elif day < 21:
        dec_start_day = '11'
    else:
        dec_start_day = '21'

    return year + month + dec_start_day

def get_dec_end(in_date):
    year = str(in_date)[0:4]
    month = str(in_date)[4:6]
    day = int(str(in_date)[6:8])

    if day < 11:
        dec_end_day = '10'
    elif day < 21:
        dec_end_day = '20'
    else:
        dec_end_day = str(getNrDays(month, year))

    return year + month + dec_end_day

def getNrDays(this_month, this_year):
    this_year = int(this_year)
    this_month = int(this_month)
    nrDaysInMonth = [31, checkLeapYear(this_year), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
    return nrDaysInMonth[this_month - 1]

def checkLeapYear(clYear):
    clYear = int(clYear)
    if (clYear % 400) == 0:
        return 29
    elif (clYear % 100) == 0:
        return 28
    elif (clYear % 4) == 0:
        return 29
    else:
        return 28

def dateToDekad(date):
    startOfDekad = get_dec_start(date)
    year = int(startOfDekad[0:4])
    month = int(startOfDekad[4:6])
    dekadInMonth = int(startOfDekad[6]) + 1  # first character of day ('0','1', or '2')

    dekadWithoutDay = (month - 1) * 3

    return (startOfDekad[2:4] + "{:02d}".format(dekadWithoutDay + dekadInMonth))

def dekadToStartDate(dekad):
    year = '20' + str(dekad)[0:2]
    dekad_nr = int(str(dekad)[2:4])
    if dekad_nr>=37:
        raise Exception("Dekad nr is above 36: %s" % dekad)
    month = "{:02d}".format(int((dekad_nr - 1)/3) +1)
    rest = dekad_nr % 3
    if rest==0:
        dd='21'
    elif rest==2:
        dd='11'
    else:
        dd='01'
    return year + month + dd

def create_list_of_dates_in_range(startDate, endDate):
    start = datetime.datetime.strptime(str(startDate), '%Y%m%d')
    end = datetime.datetime.strptime(str(endDate), '%Y%m%d')
    d = start
    d_lst=[d.strftime('%Y%m%d')]
    while d < end:
        d += datetime.timedelta(1)
        d_lst.append(d.strftime('%Y%m%d'))
    return d_lst

def get_days_in_dekad(indate):
    end = datetime.datetime.strptime(get_dec_end(indate), '%Y%m%d')
    start = datetime.datetime.strptime(get_dec_start(indate), '%Y%m%d')
    return (end - start).days +1

def get_dekad_in_month(indate):
    dekad = dateToDekad(indate)
    dekad_nr = dekad[2:4]
    return int(dekad_nr) % 3

def parse_date(indate):
    M_ASCI = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L']
    yyyy = indate[0:4]
    yy = indate[2:4]
    mm = indate[4:6]        #01-12
    m = M_ASCI[int(mm)-1]     #[A=Jan. - L=Dec.]
    tty = dateToDekad(indate)[2:4]  #[1-36]
    ttm = get_dekad_in_month(indate) #dekad_in_month [1-3]
    dd = indate[6:8]
    doy = (datetime.datetime.strptime(indate, "%Y%m%d") - datetime.datetime.strptime(yyyy +'0101', "%Y%m%d")).days +1  #day in year [1-365/6]
    dd_st = get_dec_start(indate)[6:8]  #startday of dekad [1/11/21]
    dpm = calendar.monthrange(int(yyyy),int(mm))[1]   #days in month mm [1-31]
    did = get_days_in_dekad(indate)
    return yyyy, yy, mm, m, tty, ttm,dd, doy, dd_st, dpm,did

def D50(yyyymmdd):
    #D50  = relative date, referred to 1950/01/01. D50=1 on 1950/01/01. Pro: These relative dates can be added/subtracted.
    d_end = datetime.datetime.strptime(yyyymmdd, "%Y%m%d")
    d_start = datetime.datetime.strptime(('19500101'), '%Y%m%d')
    days = d_end - d_start + datetime.timedelta(days=1)
    return days.days

def run_command(cmd):
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    err_nr = p.returncode
    return str(stdout), str(stderr), err_nr

def mask_data(in_array, mask, new_mask_val=255, pixel_convert_lst=[]):
    #mask=0 ==> Water, rest is land
    # 255 -> 251
    # 254 -> 251
    if not in_array.shape == mask.shape:
        raise Exception('Shapes are different for mask and in_array')

    mask = np.where(mask < 1, 1, 0) # set all land pixels to 1
    in_array_masked = np.ma.masked_array(in_array, mask=mask)
    for tup in pixel_convert_lst:
        old_val = tup[0]
        new_val = tup[1]
        in_array_masked = ma.where(in_array_masked == old_val, new_val, in_array_masked)
    out_array = in_array_masked.filled(new_mask_val)
    return out_array



def tif_update_metadata(fname, offset=0, scale=1, units=None, description=None, flags=None, nodata=None, processing_info=None):
    '''update file with metadata tags'''

    #create argumentlist
    args = '-mo "creator=VITO" -mo "file_creation=%s"' % datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
    #TODO : enable scale & offset again
    #if scale: args += ' -scale "%s"' % scale
    #if offset: args += ' -offset "%s"' % offset
    if units: args += ' -mo "units=%s"' % units
    if flags: args += ' -mo "flags=%s"' % flags
    if processing_info: args += ' -mo "processing_info=%s"' % (processing_info)
    if description: args += ' -mo "description=%s"' % description
    if nodata: args += ' -a_nodata %s' % nodata


    #skip if not arguments needs to be set
    if len(args)==0: return fname

    cmd = 'gdal_edit.py %s %s' % (args, fname)
    out, err, errnr = run_command(cmd)
    if errnr:
        raise Exception("Unable to run %s ---> ERR: %s; %s" % (cmd, out, err))
    return fname


def validate_inputs(in_img_list, spectral_validation=True, spatial_validation=True, datatype_should_be=None):
    bbox_list = []
    shape_list = []
    scale_list = []
    offset_list = []
    datatype_list = []
    for img in in_img_list:
        with rasterio.open(img) as src:
            shape_list.append(src.shape)
            bbox_list.append(src.bounds)
            scale_list.append(src.scales[0])
            offset_list.append(src.offsets[0])
            datatype_list.append(src.dtypes[0])
            # validating blocksize
            blockxsize = int(src.profile["blockxsize"])
            blockysize = int(src.profile["blockysize"])
            if blockxsize != meteo_settings_dict["input_blockXsize"] or blockysize != meteo_settings_dict[
                "input_blockYsize"]:
                raise Exception("Input blocksize %s,%s does not match the expected ones %s,%s"
                                % (blockxsize,
                                   blockysize,
                                   meteo_settings_dict["input_blockXsize"],
                                   meteo_settings_dict["input_blockYsize"]))

    if spectral_validation:
        scale_set = set(scale_list)
        offset_set = set(offset_list)
        datatype_set = set(datatype_list)
        if len(scale_set) != 1 or len(offset_set) != 1 or len(datatype_set) != 1:
            print("----------------------")
            print("Scales : %s" % scale_list)
            print("offsets : %s " % offset_list)
            print("datatypes : %s" % datatype_list)
            raise Exception("NOT PASSING CHECK - Spectral Validation:")
        if datatype_should_be != None:
            if list(datatype_set)[0] != datatype_should_be:
                raise Exception("Datatype mismatch!! expected : %s, result : %s" % (datatype_should_be, datatype_set[0]))

    if spatial_validation:
        # Validate boundingbox and number of rows/columns
        shape_set = set(shape_list) # because all values should be the same, only one value should be present in the set.
        bbox_set = set(bbox_list)
        if len(shape_set) != 1: raise Exception(
            'Shapes of the in images has differences : %s for files %s' % (shape_list, in_img_list))

        # check if all coordinates of the bbox are equal, keep in mind that some rounding errors can be there
        left = [];
        right = [];
        top = [];
        bottom = []
        digits = 6  # 7 digits is less than a meter (for 100m data)
        for bbox in bbox_list:
            left.append(round(bbox[0], digits))
            bottom.append(round(bbox[1], digits))
            right.append(round(bbox[2], digits))
            top.append(round(bbox[3], digits))
        left_set = set(left)
        bottom_set = set(bottom)
        right_set = set(right)
        top_set = set(top)

        # if len(bbox_set) != 1: raise Exception('Shapes of the in images has differences : %s' % bbox_list)
        if len(left_set) != 1 or len(bottom_set) != 1 or len(right_set) != 1 or len(top_set) != 1:
            raise Exception('Shapes of the in images has differences : %s' % bbox_list)

    total_number_of_blocks = round(int(shape_list[0][0]) / blockxsize * int(shape_list[0][1]) / blockysize)
    if total_number_of_blocks == 1: print("-Warn- Total number of blocks is 1! For big images this can couse memory"
                                          " issues. To solve this, provide is tiff with blocks of eg 256:256")
    return total_number_of_blocks

def check_chronological_order_input_lst(in_file, nr_of_files=0):
    #for lta,phe a chronological order is needed in the in_file. This function will check this order.
    with open(in_file, 'r') as f:
        in_file_lst = f.readlines()
    in_file_lst = list(map(lambda record: record.strip(), in_file_lst))  # remove line breaks from list
    fname_lst = list(map(lambda record: os.path.basename(record), in_file_lst))
    dekad_lst=[]
    for fname in fname_lst:
        pattern='.*_([0-9]{4}).tif'
        m = re.search(pattern, fname)
        if m is not None:
            dekad = m.group(1)
        else:
            #it is probably an LTA product
            pattern = '.*_([0-9]{2}).tif'
            m = re.search(pattern, fname)
            if m is not None:
                dekad = '62' + m.group(1)
            else:
                raise Exception("Unable to parse %s" % fname)

        dekad_lst.append(dekad)

    #we now have the dekad_lst from the file, let us compare this with a theorical dekad list that we builid on the fly
    first_dekad = dekad_lst[0]
    prev_dekad=0
    for counter in range(len(dekad_lst)):
        dekad_from_file=dekad_lst[counter][2:4]
        if int(dekad_from_file) == prev_dekad + 1:
            prev_dekad +=1
        else:
            print (dekad_lst)
            raise Exception('error in dekad_lst arround %s' % dekad_lst[counter])
        if prev_dekad == 36:
            prev_dekad = 0

    print('Checking Chronological order of inputs')
    print(' - in_file      : %s' % in_file)
    print(' - start dekad  : %s' % dekad_lst[0])
    print(' - end dekad    : %s' % dekad_lst[-1])

    #test nr of files
    if not int(nr_of_files)==0:
        #test for number of available files
        if nr_of_files != len(dekad_lst):
            raise Exception('The number of files is incorrect! is:%s, should_be:%s' %(len(dekad_lst), nr_of_files))

    #test if nr of files is for a full year
    if len(dekad_lst) % 36 > 0:
        raise Exception("some dekads are missing, the total number of files should be a multiple of 36!")
    #print(' - nr of dekads : %s' % len(dekad_lst))


def _str2bool(v):
    import argparse
    try:
        if v.lower() in ('yes', 'true', 't', 'y', '1'): return True
        if v.lower() in ('no', 'false', 'f', 'n', '0'): return False
    except:
        raise argparse.ArgumentTypeError('Wrong value in _str2bool: '+str(v))