import argparse
from pyspark import SparkContext
from pyspark import SparkConf
from datetime import datetime 
from datetime import timedelta
import json
import os
import glob
import sys

from dcs4cop_vmm_postproc.helper import Helper
from dcs4cop_vmm_postproc.helper import Dcs4cop_product
from dcs4cop_vmm_postproc import process_tile
from dcs4cop_vmm_postproc import process_tile_vmm

def parse_arguments():

    parser = argparse.ArgumentParser(prog='process_all_tiles.py')
    
    parser.add_argument("--outputRootDir",type=str,help="Output root directory",
                        required=True)
    parser.add_argument("--producttype",type=str, help="It should be s2 or l8",
                        required=True)
    parser.add_argument("--productsource",nargs='+', type=str, help="It can be a nc file or a txt file containing nc file paths)",
                        required=False)

    """ if no productsource is given, we will do a scan on disk using the filters below"""
    parser.add_argument("--inputRootDir",type=str,help="Input root directory",
                        required=False)
    parser.add_argument("--startdate", type=str, help="startdate",
                        required=False)
    parser.add_argument("--enddate", type=str, help="enddate",
                        required=False)
    parser.add_argument("--filepattern", type=str, help="filepattern: *, ?, and character ranges expressed with [] will be correctly matched.", default="*",
                        required=False)
    parser.add_argument("--client", type=str, help="client is VMM or DCS4COP",
                        required=False)


    return parser.parse_args()


def getProductsFromFileSystem(rootDir, filepattern, startdate, enddate):
    #find all products between start en enddate
    startdate = datetime.strptime(startdate, '%Y%m%d')
    enddate = datetime.strptime(enddate, '%Y%m%d')
    dateIterator = startdate
    fileLst=[]
    while dateIterator <= enddate :
        datestr = dateIterator.strftime('%Y%m%d')
        yyyy = datestr[0:4]
        mm = datestr[4:6]
        dd = datestr[6:8]
        pattern = os.path.join(rootDir,filepattern)
        lst = glob.glob(pattern)
        fileLst.extend(lst)
        dateIterator = dateIterator + timedelta(days=1)
    return fileLst


    

def main():

    args = parse_arguments()

    sc_conf = SparkConf()
    #sc_conf.set('spark.executor.memory', '4g')
    #sc_conf.set('spark.executor.cores', '2')
    #sc_conf.set('spark.cores.max', '1')
    sc = SparkContext(conf=sc_conf)



    try:
      
        """GET THE INPUTS from:
            1) a txt file
            2) one tilepath
            3) a filesystem scan
        """
        print (args.productsource)
        if len(args.productsource) > 1 :
            """this is a list of tiles"""
            print ("source : its a list of tiles")
            inputProducts=[]
            for prod in args.productsource:
                    inputProducts.append(Dcs4cop_product(prod.strip(),args.producttype, args.outputRootDir ), args.client)
        else:
         if args.productsource :
            print (args.productsource)
            print (args.productsource[0])
            print (args.productsource[0][-3:])


            sourceExtension = os.path.splitext(args.productsource[0])[1]
            if sourceExtension == '.txt':
                print ("Source : txt")
                inputProducts=[]
                f=open(args.productsource[0],'r')
                lst=f.readlines()
                f.close()
                for prod in lst:
                    inputProducts.append(Dcs4cop_product(prod.strip(),args.producttype, args.outputRootDir ))
                print ('-I- input products listed : ' + str(len(inputProducts)))
            elif sourceExtension == '.json':
                print("Source json")
                inputProducts = []
                try:
                    with open(args.productsource[0], 'r') as f:
                        contents = json.load(f)
                except Exception as e:
                    print(e)

                lst = [item.get('location') for item in contents]
                print(lst)
                for prod in lst:
                    inputProducts.append(Dcs4cop_product(prod.strip(), args.producttype, args.outputRootDir))
                print('-I- input products listed : ' + str(len(inputProducts)))
            elif sourceExtension == '.nc':
                print ("source : nc")
                inputProducts = [Dcs4cop_product(args.productsource[0], args.producttype, args.outputRootDir )]
            else:
                print ("error : " + sourceExtension)
         else:
            #get products from filesystem
            print ("source : get products from filesystem")
            inputProducts = getProductsFromFileSystem(args.inputRootDir, args.filepattern, args.startdate, args.enddate)
        
        """Filter inputs based on output product that already exists"""
        print ("-I- Start filtering")
        products = Helper.filterIfOutPutExists(inputProducts)
        numberOfProducts =len(products)
        if numberOfProducts > 0:
            products_rdd = sc.parallelize(products, len(products))
    
            if args.producttype.upper() == 'S2':
                if args.client == 'VMM':
                    products_rdd.foreach(lambda product : process_tile_vmm.s2_tile(product))
                else:
                    products_rdd.foreach(lambda product: process_tile.s2_tile(product))
            elif args.producttype.upper() == 'L8' :
                products_rdd.foreach(lambda product : process_tile.l8_scene(product))
    finally:
        sc.stop()


if __name__ == "__main__":
    main()
