import argparse
from pyspark import SparkContext
from pyspark import SparkConf
import subprocess
import os
from os.path import join
import traceback
from frame_s2.wrapper_pmax import run_pmax
from frame_s2.composite import start_composite
import frame_s2.postproc_helper as postproc_helper
from frame_s2.postproc_helper import create_quicklook
import datetime
import rasterio
import numpy as np


from frame_s2.fapar import start_fapar

#Proba Lv2 imports
from frame_s2.config.FRAME_probav_settings import proba_settings

#Lv1 processing
from frame_s2.config.FRAME_settings import lv1_settings, lv2_settings
from frame_s2.lv1_import_and_mask import start_import_and_mask_lv1
#from frame_s2.smooth15 import start_smooth15
from frame_s2.fapar import start_fapar
from frame_s2.pact import start_pact
from frame_s2.qua_adapt import start_qua_adapt
from frame_s2.nok import start_nok
from frame_s2.lta import start_lta

from frame_s2.postproc_helper import dateToDekad, getHistoricalDekadList, dekadToStartDate, dekadToStartDate, get_dec_end, tif_update_metadata

def mask(in_file, mask_file, out_file, ftype):
    with rasterio.open(mask_file, 'r', sharing=False) as src:
        profile = src.profile

        # set output
    profile.update(
        dtype=rasterio.uint8,
        count=1,
        compress='lzw',
        BIGTIFF='YES')

    print ('start processing')
    with rasterio.open(out_file, 'w+', **profile, sharing=False) as dst:
        with rasterio.open(in_file, 'r', sharing=False) as src:
            with rasterio.open(mask_file, 'r', sharing=False) as msk:
                print("Masking file %s" % out_file)
                for ji, window in dst.block_windows(1):
                    #print ('ji : %s, %s' % (ji[0], ji[1]))
                    in_array = src.read(1, window=window).astype(np.ubyte)
                    msk_array = msk.read(1, window=window).astype(np.ubyte)
                    out_array = np.where(msk_array==1, in_array, 255)
                    dst.write(out_array, 1, window=window)

    #update tif metadata
    out_file = tif_update_metadata(out_file,
                                   scale=lv2_settings[ ftype + '_SCALE'],
                                   offset=lv2_settings[ftype + '_OFFSET'],
                                   description=lv2_settings[ftype + '_DESCRIPTION'],
                                   units=lv2_settings[ftype + '_UNITS'],
                                   flags=lv2_settings[ftype + '_FLAGS']
                                   )
    print("  Finished successfully with writing %s" % out_file)


def parse_arguments():

    parser = argparse.ArgumentParser(prog='postprocess_all_lv1.py')

    parser.add_argument("--input_lst", type=str,
                        help="list of inputs(dekads/files/...) to process.",
                        required=True)
    return parser.parse_args()

def run_command(cmd):
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    err_nr = p.returncode
    return str(stdout), str(stderr), err_nr

def calculate_fapar(dekad):
    in_file = join('/vitodata/Frame/Lv2_PROBA/SSB/ndv/smo/', 'L2_SSB_NDVI_%s.tif' % dekad)
    fapar_file = join('/vitodata/Frame/Lv2_PROBA/SSB/fpr/smo/', 'L2_SSB_FPR_%s.tif' % dekad)
    start_fapar(in_file, proba_settings['FPR_POLYNOMIAL_COEFFICIENTS_PROBAV'], fapar_file)
    print ("Done processing FAPAR dekad :%s" % dekad)

def process_lv1_dekad( folder_name, root_dir ):
    os.environ['GDAL_CACHEMAX'] = lv1_settings['GDAL_CACHEMAX']
    folder_name = folder_name.strip() #in format YYYYMMDD

    ##IMPORTING
    #print ("Importing and masking = %s" % folder_name)
    #start_import_and_mask_lv1(root_dir, lv1_settings['LS_MASK_FILE'], folder_name, lv1_settings['ALB_NORMALISATION'])

    # ##SMOOTHING
    # end_date_yyyymmdd = os.path.basename(folder_name)
    # current_dekad = dateToDekad(end_date_yyyymmdd)
    # start_in_dekad = getHistoricalDekadList(current_dekad, 36)[35]
    # start_out_dekad = getHistoricalDekadList(current_dekad, 6)[5]
    # start_in_date_yyyymmdd = dekadToStartDate(start_in_dekad)
    # start_out_date_yyyymmdd = dekadToStartDate(start_out_dekad)
    # start_in_date_yyyymmdd = '20200101'
    # end_date_yyyymmdd = '20211201'
    # start_out_date_yyyymmdd = '20210101'
    # # Smoothing NDVI
    # #NDVI + QUA
    # start_smooth15( 1,   join(root_dir, 'ndv', 'msk', 'L1_NDVI_'), '', start_in_date_yyyymmdd,  end_date_yyyymmdd, 0, '0.004 -0.08 0 250',
    #                 join(root_dir, 'ndv', 'smo', 'L1_NDVI_'), '', start_out_date_yyyymmdd, end_date_yyyymmdd,
    #                 join(root_dir, 'ndv', 'quaDay', 'L1_QUAL_'),
    #                 0, 0.15, 99,  0.025)
    #
    # #ALBEDO #TODO the 200 max value kan ook 140 zijn
    # start_smooth15(0,   join(root_dir, 'alb', 'msk', 'L1_ALB_'), '', start_in_date_yyyymmdd,  end_date_yyyymmdd, 0, '0.005 0 0 200',
    #                join(root_dir, 'alb', 'smo', 'L1_ALB_'), '',  start_out_date_yyyymmdd, end_date_yyyymmdd, 0,
    #                0, 0.00, 99, 0.025)

    #FAPAR generation
    # fapar
    in_dir = join(root_dir, 'ndv', 'smo')  # previous nrt versions(until currentdekad-5) will be overwritten
    current_dekad = 2134
    for dekad in getHistoricalDekadList(current_dekad, 6):
        in_file = join(in_dir, 'L1_NDVI_%s.tif' % dekad)
        fapar_file = join(root_dir, 'fpr', 'smo', 'L1_FPR_%s.tif' % dekad)
        start_fapar(in_file, lv1_settings['FPR_POLYNOMIAL_COEFFICIENTS_MODIS'], fapar_file)

def merge_SSB_AFR_ALG_into_LTA(dekad):
    dekad = dekad.strip()  #strip potential newlines
    os.environ['GDAL_CACHEMAX'] = '1000000000'

    #NDVI
    cmd = "gdal_merge.py  --config GDAL_CACHEMAX 512 -o /vitodata/Frame/Lv2_PROBA/SSB/ndv/smo/L2_SSB_NDVI_%s.tif -co " \
          "'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' " \
          "/vitodata/frame_7/Lv2_SSB/ndv/smo/L2_SSB_NDVI_%s.img " \
          "/vitodata/Frame/Lv2_PROBA/CAV/ndv/smo/L2_CAV_NDVI_%s.tif" % (dekad,dekad,dekad )
    print (cmd)
    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        print(traceback.print_exc())
        print(stdout)
        print(stderr)
        raise Exception("Converting NDVI img to tif failed for dekad %s" % dekad )

def merge_CAV_into_SSB( record ):
    dekad = record.strip()  #strip potential newlines
    os.environ['GDAL_CACHEMAX'] = '1000000000'

    #NDVI
    # cmd = "gdal_merge.py  --config GDAL_CACHEMAX 512 -o /vitodata/Frame/Lv2_PROBA/SSB/ndv/smo/L2_SSB_NDVI_%s.tif -co " \
    #       "'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' " \
    #       "/vitodata/frame_7/Lv2_SSB/ndv/smo/L2_SSB_NDVI_%s.img " \
    #       "/vitodata/Frame/Lv2_PROBA/CAV/ndv/smo/L2_CAV_NDVI_%s.tif" % (dekad,dekad,dekad )
    # print (cmd)
    # stdout, stderr, err_nr = run_command(cmd)
    # if err_nr:
    #     print(traceback.print_exc())
    #     print(stdout)
    #     print(stderr)
    #     raise Exception("Converting NDVI img to tif failed for dekad %s" % dekad )
    #
    #
    # #ALB
    # cmd = "gdal_merge.py  --config GDAL_CACHEMAX 512 -o /vitodata/Frame/Lv2_PROBA/SSB/alb/smo/L2_SSB_ALBEDO_%s.tif " \
    #     "-co 'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' " \
    #     "/vitodata/frame_7/Lv2_SSB/alb/smo/L2_SSB_ALB_%s.img " \
    #     "/vitodata/Frame/Lv2_PROBA/CAV/alb/smo/L2_CAV_ALB_%s.tif" % (dekad,dekad,dekad )
    # print(cmd)
    # stdout, stderr, err_nr = run_command(cmd)
    # if err_nr:
    #     print(traceback.print_exc())
    #     print(stdout)
    #     print(stderr)
    #     raise Exception("Converting ALB img to tif failed for dekad %s" % dekad)

    #QUA
    #cmd = "gdal_translate -a_srs 'EPSG:4326' -co 'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' /vitodata/frame_3/Lv2_CAV/ndv/qua_Bart/L2_CAV_QUAL_%s.img /vitodata/Frame/Lv2_PROBA/SSB/ndv/quaCAV.orig/L2_CAV_QUAL_%s.tif" % (dekad,dekad )
    # cmd = 'gdalbuildvrt -srcnodata "254 255" -o /vitodata/Frame/Lv2_PROBA/SSB/ndv/qua/L2_SSB_QUAL_%s.vrt /vitodata/Frame/Lv2_PROBA/SSB/ndv/qua.orig/L2_SSB_QUAL_%s.tif /vitodata/Frame/Lv2_PROBA/SSB/ndv/quaCAV.orig/L2_CAV_QUAL_%s.tif'  % (dekad,dekad,dekad )
    # print(cmd)
    # stdout, stderr, err_nr = run_command(cmd)
    # if err_nr:
    #     print(traceback.print_exc())
    #     print(stdout)
    #     print(stderr)
    #     raise Exception("Converting QUA img to tif failed for dekad %s" % dekad)
    cmd = "gdal_translate -a_nodata '-32768' -co 'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' /vitodata/frame_7/Lv2_SSB/met/npm/s10/L2_SSB_NPM_%s.img /vitodata/Frame/Lv2_PROBA/SSB/met/npm/s10/L2_SSB_NPM_%s.tif" % (dekad,dekad )
    # # cmd = "gdal_merge.py --config GDAL_CACHEMAX 512 -o /vitodata/Frame/Lv2_PROBA/SSB/ndv/qua/L2_SSB_QUAL_%s.tif -co " \
    # #        "'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' -n 255 " \
    # #        "/vitodata/Frame/Lv2_PROBA/SSB/ndv/qua.orig/L2_SSB_QUAL_%s.tif " \
    # #        "/vitodata/frame_3/Lv2_CAV/ndv/qua_Bart/L2_CAV_QUAL_%s.img" % (dekad,dekad,dekad )
    print(cmd)
    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        print(traceback.print_exc())
        print(stdout)
        print(stderr)
        raise Exception("Converting NPM10 img to tif failed for dekad %s" % dekad)
    # #

    print ("Done processing dekad :%s" % dekad)

def process_dekad_npm10(dekad):
    dekad=dekad.strip()
    print (dekad)
    outfile = '/vitodata/Frame/S2/nrt/meteo/ALG/npm/s10/L2_ALG_NPM_%s.tif' % dekad
    #outfile = '/vitodata/Frame/Lv1/meteo/AFR/npm/s10/L1_AFR_NPM_%s.tif' % dekad
    dekad_startdate = dekadToStartDate(dekad)
    dekad_endddate = get_dec_end(dekad_startdate)[2:]
    datestr = dekad_startdate[2:]


    infile_lst = []
    while datestr <= dekad_endddate:
        print (' %s' % datestr)
        in_file = '/vitodata/Frame/S2/nrt/meteo/ALG/npm/s01/L2_ALG_NPM_%s.tif' % datestr
        infile_lst.append(in_file)
        datestr = (datetime.datetime.strptime(datestr,'%y%m%d') + datetime.timedelta(days=1)).strftime('%y%m%d')

    start_composite(infile_lst, outfile, in_nodata=-1,
                    out_nodata=-32768,
                    out_meta_info='MEAN-value Mosaic',
                    out_meta_units='gC/m2/day' )

def clip_from_proba(dekad):
    #this function is used for clipping an extent from the PROBA REPO(full Africa)
    dekad=dekad.strip()
    print (dekad)

    roi='ALG'
    in_file_extention = 'img'
    in_folder = '/vitodata/frame_3/lv2'
    in_alb = join(in_folder,'alb','smo','L2_ALB_%s.%s') % (dekad, in_file_extention)
    in_ndv = join(in_folder,'ndv','smo','L2_NDVI_%s.%s') % (dekad, in_file_extention)
    in_qua = join(in_folder,'ndv','qua','L2_QUAL_%s.%s') % (dekad, in_file_extention)
    in_fpr = join(in_folder,'fpr','smo','L2_FPR_%s.%s') % (dekad, in_file_extention)
    in_npp = join(in_folder,'npp','L2_NPP_%s.%s') % (dekad, in_file_extention)
    #in_files=[in_alb, in_ndv, in_qua, in_npp]
    in_files=[in_alb, in_ndv, in_qua, in_fpr]

    out_folder = '/vitodata/Frame/Lv2_PROBA/ALG'
    mask_file = '/vitodata/Frame/S2/DEV/nrt/refs/from_ELEAF/L2_ALG_mask.tif'   #this should be the mask from Eleaf
    tmp_dir = '/vitodata/Frame/Lv2_PROBA/tmp'
    for in_file in in_files:
        fname = os.path.basename(in_file)
        fname = fname.replace('L2_','L2_ALG_').replace(".img",'.tif')   #always replace img by tif

        #tmp_file = join(tmp_dir, fname)
        #out_file = join(out_folder, fname)
        nodata='-a_nodata 255'
        if fname[-10:-9] == 'I':
            ftype='NDV'
            tmp_file = join(tmp_dir, 'ndv','smo',fname)
            out_file = join(out_folder, 'ndv','smo',fname)
        if fname[-10:-9] == 'B':
            ftype='ALB'
            tmp_file = join(tmp_dir, 'alb','smo',fname)
            out_file = join(out_folder, 'alb','smo',fname)
        if fname[-10:-9] == 'L':
            ftype='QUA'
            tmp_file = join(tmp_dir, 'ndv','qua',fname)
            out_file = join(out_folder, 'ndv','qua',fname)
        if fname[-10:-9] == 'R':
            ftype='FPR'
            tmp_file = join(tmp_dir, 'fpr','smo',fname)
            out_file = join(out_folder, 'fpr','smo',fname)
        if fname[-10:-9] == 'P':
            ftype='NPP'
            tmp_file = join(tmp_dir, 'npp',fname)
            out_file = join(out_folder, 'npp',fname)
            nodata='-a_nodata -9999'


        #clip data to a smaller extent)
        if roi == 'ALG':
            extent = '-projwin -2.600694460483993 37.600694458884000 9.1701389 31.9995040'

        if roi in ['ALG', 'SSB']:
            cmd = "gdal_translate %s -a_srs 'EPSG:4326' -tr 0.000992063492000 -0.000992063492000 %s -co 'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' %s %s" % (nodata, extent, in_file, tmp_file)
            print ('Start clipping %s ' %tmp_file)
            stdout, stderr, err_nr = run_command(cmd)
            if err_nr:
                raise Exception("Error while executing %s\n%s\n%s" % (cmd, stdout, stderr))
        else:
            tmp_file = in_file

        mask(tmp_file, mask_file, out_file, ftype)

        #create quicklook for ndvi and npp
        if ftype in ['NDV', 'NPP']:
            #create quicklook
            ql_width = lv2_settings['ROIS'][roi]['ql_width']
            ql_height = lv2_settings['ROIS'][roi]['ql_height']
            create_quicklook(out_file, 'ndvi.txt', height=ql_height, width=ql_width)

def from_img_2_tif(dekad):
    os.environ['GDAL_CACHEMAX'] = '1000000000'
    in_dir = '/vitodata/frame_3/BIOPAR/EUR/ACT/IMG'
    out_dir = '/data/Mars/MEP/biopar/EUR/ACT/IMG'
    in_img = join(in_dir, 'ct%sb.img' % dekad)
    out_tif = join(out_dir, 'ct%sb.tif' % dekad)

    #TODO change for FRAME again
    cmd = "gdal_translate -a_srs 'EPSG:3035' -co 'BLOCKXSIZE=256' -co 'BLOCKYSIZE=256' -co 'TILED=YES' -co 'COMPRESS=LZW' %s %s" % ( in_img, out_tif)

    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        raise Exception("Error while executing %s\n%s\n%s" % (cmd, stdout, stderr))
def nok_single_run(dummy):
    #use the mask with 0(water) and 1-255(keep)!!!!
    ###'SRI'
    # in_file_lst_file = '/vitodata/Frame/S2/nrt/annual/SRI/nok/2020/qua_inputs.lst'
    # in_mask = '/vitodata/Frame/S2/nrt/refs/clipped_masks/S2_SRI_mask.tif'
    # out_nok = '/vitodata/Frame/S2/nrt/annual/SRI/nok/2020/L2_SRI_Nok_2015_2020_F0.tif'
    #####'AFR'
    in_file_lst_file = '/vitodata/Frame/S2/nrt/annual/AFR_without_ALG_SSB/nok/2021/qua_inputs.lst'
    in_mask = '/vitodata/Frame/S2/nrt/refs/S2_AFR_without_SSB_ALG_mask.tif'
    out_nok = '/vitodata/Frame/S2/nrt/annual/AFR_without_ALG_SSB/nok/2021/L2_AFR_Nok_2015_2021_F0.tif'

    #####SSB
    in_file_lst_file = '/vitodata/Frame/S2/nrt/annual/SSB/nok/2021/qua_inputs.lst'
    in_mask = '/vitodata/Frame/S2/nrt/refs/clipped_masks/S2_SSB_mask_clipped.tif'
    out_nok = '/vitodata/Frame/S2/nrt/annual/SSB/nok/2021/L2_SSB_Nok_2015_2021_F0.tif'

    #####ALG
    in_file_lst_file = '/vitodata/Frame/S2/nrt/annual/ALG/nok/2020/qua_inputs.lst'
    in_mask = '/vitodata/Frame/S2/DEV/nrt/refs/from_ELEAF/L2_ALG_mask.tif'
    out_nok = '/vitodata/Frame/S2/nrt/annual/ALG/nok/2020/L2_ALG_Nok_2015_2020_F0.tif'

    #general settings
    in_Vlo = 0
    in_Vhi = 240
    out_flag_masked_pixels = 255
    out_flag_all_flagged_in = 0
    calculation_type = 2
    lo_val = 0
    hi_val = 0

    out_datatype = 1
    count_ins = 0
    start_nok(in_file_lst_file, in_mask, in_Vlo, in_Vhi, out_flag_masked_pixels, out_flag_all_flagged_in, out_nok, calculation_type, lo_val, hi_val, out_datatype, count_ins)
def lta_single_run(dummy):
    year = '2021'
    roi = 'ALG'

    in_lst = '/vitodata/Frame/S2/nrt/annual/%s/lta/%s/ndv_smoothed.lst' % (roi, year)
    out_folder= '/vitodata/Frame/S2/nrt/annual/%s/lta/%s/' % (roi, year)
    out_file_suffix = 'L2_%s_LTA_<dek>' % (roi)

    OUT_SCALE=0.004
    OUT_OFFSET=-0.08
    OUT_UNITS='-'

    start_lta(in_lst, out_folder, out_file_suffix, OUT_SCALE, OUT_OFFSET, OUT_UNITS)

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)


    if not os.path.exists(args.input_lst):
        raise Exception("The input folder list does not exist : %s" % args.input_lst)
    try:
        with open(args.input_lst) as f:
            in_records = f.readlines()
        in_records = list(map(lambda dekad: dekad.strip(), in_records))  # strip each dekad
        number_of_files = len(in_records)
        if number_of_files > 0:
            #logdir = os.path.join(args.output_dir, 'logs')
            tiles_rdd = sc.parallelize(in_records, len(in_records))
            tiles_rdd.foreach(lambda record: from_img_2_tif(record))

            ### ANNUAL processing , use wrap_per_year instead!!!!
            #tiles_rdd.foreach(lambda record: nok_single_run('dummy'))   #NOK
            #tiles_rdd.foreach(lambda record: lta_single_run('dummy'))   #LTA


            ##NPM1
            #tiles_rdd.foreach(lambda yyyymmdd: run_pmax('/vitodata/Frame/S2/nrt/meteo/ALG/', 'ALG', yyyymmdd, level = 2, period = 1))
            #tiles_rdd.foreach(lambda yyyymmdd: run_pmax('/vitodata/Frame/Lv1/meteo/AFR/', 'AFR', yyyymmdd, level = 1, period = 1))
            ##NPM10
            #tiles_rdd.foreach(lambda record: process_dekad_npm10(record))

            #NPP for S2
            # tiles_rdd.foreach(lambda dekad: start_pact('/vitodata/Frame/S2/nrt/post/ALG/fpr/smo', '/vitodata/Frame/S2/nrt/meteo/ALG/npm/s10','/vitodata/Frame/S2/nrt/refs/LU2015_ALG.tif',
            #                                            'LU_RUE.txt', '/vitodata/Frame/S2/nrt/post/ALG/npp',
            #                                            dekad, 'ALG', out_nodata=-9999, level=2))
            #NPP 4 proba
            # tiles_rdd.foreach(lambda dekad: start_pact('/vitodata/Frame/Lv2_PROBA/ALG/fpr/smo', '/vitodata/Frame/S2/nrt/meteo/ALG/npm/s10','/vitodata/Frame/S2/nrt/refs/LU2015_ALG.tif',
            #                                            'LU_RUE.txt', '/vitodata/Frame/Lv2_PROBA/ALG/npp',
            #                                            dekad, 'ALG', out_nodata=-9999, level=2))

            #Clip extent out of PROBA data
            #tiles_rdd.foreach(lambda record: clip_from_proba(record))


        else:
            raise Exception("input list is empty %s" % args.input_lst )

    finally:
        sc.stop()


if __name__ == "__main__":
    #clip_from_proba("1936")
    main()

