import argparse
from pyspark import SparkContext
from pyspark import SparkConf
import os
from os.path import join
import json

from frame_s2.composite import start_composite


def parse_arguments():

    parser = argparse.ArgumentParser(prog='composite.py')

    parser.add_argument("--root_dir", type=str, help="root directory",
                        required=True)
    parser.add_argument("--date_list_files", type=str, help="a file with dates to process",
                        required=True)
    parser.add_argument("--level", type=int, help="1:Lv1, 2:Lv2",
                        required=False,
                        default=2)
    # parser.add_argument("--period", type=str, help="period: 1, 10",
    #                     required=False,
    #                     default=1)
    return parser.parse_args()


def run_composite(meteo_root_dir, roi, dekad, s1_in_lst, level):
    in_dir = join(meteo_root_dir, roi, 'npm/s01/')
    out_file = join(meteo_root_dir, roi, 'npm/s10/','L%s_%s_NPM_%s.tif') % (level, roi, dekad)

    #create list of inputfiles
    s1s=[]
    for s1 in s1_in_lst:
        s1s.append(join(in_dir, s1['filename']))

    #start processing
    start_composite(s1s,
                    out_file,
                    in_nodata=-1,
                    out_nodata=-32768,
                    out_meta_info='MEAN-value Mosaic',
                    out_meta_units='gC/m2/day')

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)

    try:
        with open(args.date_list_files) as f:
            s10lst = json.load(f)

        #validate json to detect & prevent doubles
        detection_lst=[]
        for s10 in s10lst:
            detection_lst.append(s10['roi'] + '_' + s10['dekad'])
        detection_set = set(detection_lst)
        if len(detection_set) < len(detection_lst):
            print("-W- Some double s10 items are filtered from this list!!!")
            s10lst = list(set(s10lst)) #fitler double items by putting them in a set, convert to a list afterwards

        number_of_jobs = len(s10lst)

        if number_of_jobs > 0:
            pmax_rdd = sc.parallelize(s10lst, len(s10lst))
            pmax_rdd.foreach(lambda s10: run_composite(args.root_dir, s10['roi'], s10['dekad'], s10['inputs'], args.level))

    finally:
        sc.stop()


if __name__ == "__main__":
    main()

